pref("extensions.deejayd-webui.server_list", "");
pref("extensions.deejayd-webui.current_server", "");
pref("extensions.deejayd-webui.use_default_refresh", true);
pref("extensions.deejayd-webui.refresh", 10);
pref("extensions.deejayd-webui.default_refresh", 10);
