# -*- coding: utf-8 -*-
# Deejayd, a media player daemon
# Copyright (C) 2007-2009 Mickael Royer <mickael.royer@gmail.com>
#                         Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

songlibrary = [
{"media_id":1, 'filename': 'rock/Dolly - Dolly/01-dolly-je_n_veux_pas_rester_sage.ogg',
 'artist': 'Dolly', 'album': 'Dolly', 'title': "Je n'veux pas rester sage"},
{"media_id":2, 'filename': 'rap/MC Solar/LTDLQ/04-mc_solaar-onzième_commandement.ogg',
 'artist': 'MC Solar', 'album': 'Le tour de la question CD1',
 'title': "Onzième commandement", 'track': '04/20'},
{"media_id":3, 'filename': 'dance/singles/Ultra Nate - Free.mp3',
 'artist': 'Ultra Nate', 'title': "Free"}
]

sample_genres = ('Country', 'Disco', 'Rock', 'Trance', 'R&B', 'Big beat', )


# vim: ts=4 sw=4 expandtab
