#playing-control div.control-button {
    padding: 0px 8px;
}

#volume-slider, #volume-widget div.volume-button {
    position: relative;
    left: 18px;
}
#volume-widget div.volume-button { top: 12px; }

#mode-extra-content div.library-item div.directory {
    position: relative; top: -10px;
}
