/*
 * An example if you want to style the tree
 */
/*
treechildren::-moz-tree-row(selected) { background-color: #ddd; }
treechildren::-moz-tree-row(focus,selected) { background-color: #aaa; }
treechildren::-moz-tree-row(odd) { background-color: #eee; }
treechildren::-moz-tree-row(odd,selected) { background-color: #ddd; }
treechildren::-moz-tree-row(focus,odd,selected) { background-color: #aaa; }
*/

/*
 * General parms
 */
#main{margin: 10px;}
caption{font-weight: bold;}


/*
 * Icons
 */
.collapsed{list-style-image: url("./images/collapsed.png");}
.expanded{list-style-image: url("./images/expanded.png");}

.audio-item{list-style-image: url("./images/audio.png");}
.directory-item{list-style-image: url("./images/folder.png");}
.video-item{list-style-image: url("./images/video.png");}
.playlist-item{list-style-image: url("./images/playlist.png");}
.static-playlist-item{list-style-image: url("./images/playlist.png");}
.magic-playlist-item{list-style-image: url("./images/magic-playlist.png");}
.webradio-item{list-style-image: url("./images/webradio.png");}
.dvd-item{list-style-image: url("./images/dvd.png");}

.add-action{list-style-image: url("./images/add.png");}
.remove-action{list-style-image: url("./images/remove.png");}
.play-action{list-style-image: url("./images/play-low.png");}
.search-action{list-style-image: url("./images/search.png");}
.edit-action{list-style-image: url("./images/edit.png");}

.shuffle-button{list-style-image: url("./images/update.png");}
.save-button{list-style-image: url("./images/save.png");}
.clear-button{list-style-image: url("./images/clear.png");}
.cancel-button{list-style-image: url("./images/cancel.png");}
.fullscreen-button{list-style-image: url("./images/fullscreen.png");}
.update-button{list-style-image: url("./images/update.png");}
.subtitle-button{list-style-image: url("./images/subtitle.png");}

#random-button{list-style-image: url("./images/shuffle.png");}
#repeat-button{list-style-image: url("./images/repeat.png");}
.goCurSong-button{list-style-image: url("./images/go.png");}

.home-menu{list-style-image: url("./images/home.png");}
.search-menu{list-style-image: url("./images/search.png");}
.playlist-menu{list-style-image: url("./images/playlist.png");}

/*
 * Example to change the style of a checked button
 */
button[checked="true"] { font-weight: bold; }
toolbarbutton[checked="true"] { font-weight: bold; }

/*
 * Playlist
 */
treechildren::-moz-tree-row(dragged) { border-top: 2px solid #333; }

/*
 * video directoey
 */
treechildren::-moz-tree-image(folder) {
list-style-image: url("./images/tree-folder.png");
padding-right: 6px;
}
treechildren::-moz-tree-image(videodir-select) {
list-style-image: url("./images/go.png");
}

/*
 * Command
 */
#command-panel{
    margin: 5px 10px 2px 20px;
    padding: 5px;
    }
#seekbar-button{max-height: 26px !important; }
#volseek-toolbar{margin: 5px 0px 5px 0px;}

/*
 * Message
 */
#message-box{
    padding: 3px;
    height:20px;}
.error{
    background-color: #EF9398;
    border: 1px solid #DC5757;
    }
.confirmation{
    background-color: #A6EF7B;
    border: 1px solid #76C83F;
    }

/*
 * Player
 */
.play-button {list-style-image: url("./images/play.png");}
.pause-button {list-style-image: url("./images/pause.png");}
.stop-button {list-style-image: url("./images/stop.png");}
.previous-button {list-style-image: url("./images/previous.png");}
.next-button {list-style-image: url("./images/next.png");}

#current-media{margin: 10px 5px 0px 5px;}
#media-info .cursong {margin: 0px;}
#current-artist,#current-url,#current-album{
    font-size: 11px;
    padding-left: 10px;
    }
#current-artist,#current-url{font-weight: bold;}
#current-album{font-style: italic;}

#seekbar{margin-top: 5px;}

treechildren::-moz-tree-row(play) {background-color: #ccc;}
treechildren::-moz-tree-image(play) {
list-style-image: url("./images/tree-play.png");
}
treechildren::-moz-tree-row(pause) {background-color: #ccc;}
treechildren::-moz-tree-image(pause) {
list-style-image: url("./images/tree-pause.png");
}

/*
 * Search
 */
#search-box{height: 11px;}

/*
 * Playlist
 */
#navigation-path {margin: 1px 10px 1px 10px;}
#navigation-path toolbarbutton{
    font-size: 10px;
    max-width:60px;
    margin: 2px;
    background-color: #ddd;
    border-left: 1px solid #ddd;
    border-right: 1px solid #ddd;
    }

/*
 * Queue
 */
#queue-panel{
height:30px;
min-height:30px;
max-height:30px;
}

/*
 * Panel
 */
#panel-select-box .listitem-title { font-weight: bold; }
.list-all, .list-unknown { font-weight: bold; }

treechildren::-moz-tree-image(static-playlist-item) {
list-style-image: url("./images/playlist.png");
}
treechildren::-moz-tree-image(magic-playlist-item) {
list-style-image: url("./images/magic-playlist.png");
}
.playlist-new {list-style-image: url("./images/playlist-new.png");}

#panel-pls-list treechildren::-moz-tree-row(dragged) {
border-top: 0px solid #333;
background-color: #ded;
}
